@@ -20,7 +20,6 @@ from eventlet import greenthread
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
-import six
 
 from cinder import exception
 from cinder.i18n import _
@@ -52,7 +51,7 @@ class StorwizeSVCReplication(object):
         except Exception as e:
             msg = (_('Unable to fail-over the volume %(id)s to the '
                    'secondary back-end, error: %(error)s') %
-                   {"id": vref['id'], "error": six.text_type(e)})
+                   {"id": vref['id'], "error": str(e)})
             LOG.exception(msg)
             raise exception.VolumeDriverException(message=msg)
 
@@ -68,7 +67,7 @@ class StorwizeSVCReplication(object):
             except Exception as e:
                 msg = (_('Unable to fail-back the volume: %(vol)s to the '
                          'master back-end, error: %(error)s') %
-                       {"vol": volume['name'], "error": six.text_type(e)})
+                       {"vol": volume['name'], "error": str(e)})
                 LOG.exception(msg)
                 raise exception.VolumeDriverException(message=msg)
 
@@ -98,20 +97,23 @@ class StorwizeSVCReplicationGlobalMirror(StorwizeSVCReplication):
 
         target_vol_name = storwize_const.REPLICA_AUX_VOL_PREFIX + vref['name']
         try:
-            attr = self.target_helpers.get_vdisk_attributes(target_vol_name)
-            if not attr:
-                opts = self.driver._get_vdisk_params(vref['volume_type_id'])
-                pool = self.target.get('pool_name')
-                src_attr = self.driver._helpers.get_vdisk_attributes(
-                    vref['name'])
-                opts['iogrp'] = src_attr['IO_group_id']
+            opts = self.driver._get_vdisk_params(vref['volume_type_id'])
+            pool = self.target.get('pool_name')
+            src_attr = self.driver._helpers.get_vdisk_attributes(
+                vref['name'])
+            opts['iogrp'] = src_attr['IO_group_id']
+            try:
                 self.target_helpers.create_vdisk(target_vol_name,
-                                                 six.text_type(vref['size']),
+                                                 str(vref['size']),
                                                  'gb', pool, opts)
+            except exception.VolumeBackendAPIException as excp:
+                if "CMMVC6035E" in excp.msg:
+                    LOG.info('Target Volume: %(vol)s already exists',
+                             {'vol': target_vol_name})
 
-            system_info = self.target_helpers.get_system_info()
+            target_system_id = self.driver._aux_state['system_id']
             self.driver._helpers.create_relationship(
-                vref['name'], target_vol_name, system_info.get('system_id'),
+                vref['name'], target_vol_name, target_system_id,
                 self.asyncmirror)
         except Exception as e:
             msg = (_("Unable to set up mirror mode replication for %(vol)s. "
@@ -167,85 +169,97 @@ class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
                     new_type['id'], volume_type=new_type)
             src_attr = self.driver._helpers.get_vdisk_attributes(
                 vref['name'])
-            # Create source change volume if it doesn't exist
-            src_change_attr = self.driver._helpers.get_vdisk_attributes(
-                source_change_vol_name)
-            if not src_change_attr:
-                src_change_opts = self.driver._get_vdisk_params(
-                    vref['volume_type_id'])
-                src_change_opts['iogrp'] = src_attr['IO_group_id']
-                # Change volumes would usually be thin-provisioned
-                src_change_opts['autoexpand'] = True
-                src_change_pool = src_attr['mdisk_grp_name']
-                if new_type:
-                    src_child_pool = (
-                        new_type_opts['storwize_svc_src_child_pool'])
-                else:
-                    src_child_pool = (
-                        src_change_opts['storwize_svc_src_child_pool'])
-                if src_child_pool:
-                    src_change_pool = src_child_pool
+            # Source change volume creation
+            src_change_opts = self.driver._get_vdisk_params(
+                vref['volume_type_id'])
+            src_change_opts['iogrp'] = src_attr['IO_group_id']
+            # Change volumes would usually be thin-provisioned
+            src_change_opts['autoexpand'] = True
+            src_change_pool = src_attr['mdisk_grp_name']
+            if new_type:
+                src_child_pool = (
+                    new_type_opts['storwize_svc_src_child_pool'])
+            else:
+                src_child_pool = (
+                    src_change_opts['storwize_svc_src_child_pool'])
+            if src_child_pool:
+                src_change_pool = src_child_pool
+            try:
                 self.driver._helpers.create_vdisk(source_change_vol_name,
-                                                  six.text_type(vref['size']),
+                                                  str(vref['size']),
                                                   'gb',
                                                   src_change_pool,
                                                   src_change_opts)
-            # Create target volume if it doesn't exist
-            target_attr = self.target_helpers.get_vdisk_attributes(
-                target_vol_name)
-            if not target_attr:
-                target_opts = self.driver._get_vdisk_params(
-                    vref['volume_type_id'])
-                target_pool = self.target.get('pool_name')
-                target_opts['iogrp'] = src_attr['IO_group_id']
+            except exception.VolumeBackendAPIException as excp:
+                if "CMMVC6035E" in excp.msg:
+                    msg = ('Source change volume: %s already exists'
+                           % source_change_vol_name)
+                    LOG.info(msg)
+
+            # Target volume creation
+            target_opts = self.driver._get_vdisk_params(
+                vref['volume_type_id'])
+            target_pool = self.target.get('pool_name')
+            target_opts['iogrp'] = src_attr['IO_group_id']
+            try:
                 self.target_helpers.create_vdisk(target_vol_name,
-                                                 six.text_type(vref['size']),
+                                                 str(vref['size']),
                                                  'gb',
                                                  target_pool,
                                                  target_opts)
+            except exception.VolumeBackendAPIException as excp:
+                if "CMMVC6035E" in excp.msg:
+                    msg = ('Target Volume: %s already exists'
+                           % target_vol_name)
+                    LOG.info(msg)
 
-            # Create target change volume if it doesn't exist
-            target_change_attr = self.target_helpers.get_vdisk_attributes(
-                target_change_vol_name)
-            if not target_change_attr:
-                target_change_opts = self.driver._get_vdisk_params(
-                    vref['volume_type_id'])
-                target_change_pool = self.target.get('pool_name')
-                if new_type:
-                    target_child_pool = (
-                        new_type_opts['storwize_svc_target_child_pool'])
-                else:
-                    target_child_pool = (
-                        target_change_opts['storwize_svc_target_child_pool'])
-                if target_child_pool:
-                    target_change_pool = target_child_pool
-                target_change_opts['iogrp'] = src_attr['IO_group_id']
-                # Change Volumes would usually be thin-provisioned
-                target_change_opts['autoexpand'] = True
+            # Target change volume creation
+            target_change_opts = self.driver._get_vdisk_params(
+                vref['volume_type_id'])
+            target_change_pool = self.target.get('pool_name')
+            if new_type:
+                target_child_pool = (
+                    new_type_opts['storwize_svc_target_child_pool'])
+            else:
+                target_child_pool = (
+                    target_change_opts['storwize_svc_target_child_pool'])
+            if target_child_pool:
+                target_change_pool = target_child_pool
+            target_change_opts['iogrp'] = src_attr['IO_group_id']
+            # Change Volumes would usually be thin-provisioned
+            target_change_opts['autoexpand'] = True
+            try:
                 self.target_helpers.create_vdisk(target_change_vol_name,
-                                                 six.text_type(vref['size']),
+                                                 str(vref['size']),
                                                  'gb',
                                                  target_change_pool,
                                                  target_change_opts)
+            except exception.VolumeBackendAPIException as excp:
+                if "CMMVC6035E" in excp.msg:
+                    msg = ('Target Change Volume: %s already exists'
+                           % target_change_vol_name)
+                    LOG.info(msg)
 
-            system_info = self.target_helpers.get_system_info()
+            target_system_id = self.driver._aux_state['system_id']
             # Get cycle_period_seconds
             src_change_opts = self.driver._get_vdisk_params(
                 vref['volume_type_id'])
             cycle_period_seconds = src_change_opts.get('cycle_period_seconds')
-            self.driver._helpers.create_relationship(
-                vref['name'], target_vol_name, system_info.get('system_id'),
+            rc_name = self.driver._helpers.create_relationship(
+                vref['name'], target_vol_name, target_system_id,
                 self.asyncmirror, True, source_change_vol_name,
                 cycle_period_seconds)
             # Set target change volume
             self.target_helpers.change_relationship_changevolume(
-                target_vol_name, target_change_vol_name, False)
+                target_vol_name, target_change_vol_name, False,
+                rc_name)
             # Start gmcv relationship
-            self.driver._helpers.start_relationship(vref['name'])
+            self.driver._helpers.start_relationship(vref['name'],
+                                                    rcrel=rc_name)
         except Exception as e:
             msg = (_("Unable to set up gmcv mode replication for %(vol)s. "
                      "Exception: %(err)s.") % {'vol': vref['id'],
-                                               'err': six.text_type(e)})
+                                               'err': str(e)})
             LOG.exception(msg)
             raise exception.VolumeDriverException(message=msg)
         LOG.debug('leave: volume_replication_setup:volume %s', vref['name'])
@@ -291,7 +305,7 @@ class StorwizeSVCReplicationManager(object):
                         return processutils.ssh_execute(
                             ssh, command, check_exit_code=check_exit_code)
                     except Exception as e:
-                        LOG.error(six.text_type(e))
+                        LOG.error(str(e))
                         last_exception = e
                         greenthread.sleep(random.randint(20, 500) / 100.0)
                 try:
